<?php

declare(strict_types=1);

namespace Drupal\oktaplus\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\Queue\QueueDatabaseFactory;
use Drupal\Core\Queue\QueueInterface;
use Drupal\oktaplus\Form\OktaplusSettingsForm;
use Drupal\oktaplus\Plugin\QueueWorker\OktaplusEventQueue;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Event hook controller.
 */
final class EventHookController extends ControllerBase {

  use AutowireTrait;

  protected readonly QueueInterface $queue;

  public function __construct(
    ConfigFactoryInterface $configFactory,
    protected readonly QueueDatabaseFactory $queueFactory,
  ) {
    $this->configFactory = $configFactory;
    $this->queue = $this->queueFactory->get(OktaplusEventQueue::PLUGIN_ID);
  }

  /**
   * One-time verification request.
   *
   * @see https://developer.okta.com/docs/concepts/event-hooks/#one-time-verification-request
   */
  public function handleVerification(Request $request): JsonResponse {
    $challenge = $request->headers->get('x-okta-verification-challenge');
    if ($challenge === NULL) {
      throw new BadRequestHttpException('Missing Okta verification challenge.');
    }
    return new JsonResponse(['verification' => $challenge]);
  }

  /**
   * Handle Okta event hook.
   */
  public function handleEvent(Request $request): JsonResponse {
    if (!$this->isAuthorizationValid($request)) {
      throw new BadRequestHttpException('Invalid authorization.');
    }

    // Add events to queue.
    $events = $this->getEventsFromRequest($request);
    foreach ($events as $event) {
      $this->queue->createItem($event);
    }

    // https://developer.okta.com/docs/concepts/event-hooks/#your-service-s-responses-to-event-delivery-requests
    return new JsonResponse([], Response::HTTP_NO_CONTENT);
  }

  /**
   * Validate request based on Authorization header configured in Okta.
   *
   * @see https://developer.okta.com/docs/guides/common-hook-set-up-steps/nodejs/main/#http-header-basic-authentication`
   */
  private function isAuthorizationValid(Request $request): bool {
    $authorization = $request->headers->get('Authorization');
    if ($authorization === NULL) {
      return FALSE;
    }
    $config = $this->config(OktaplusSettingsForm::CONFIG_NAME);
    $secret = $config->get('event_hook_secret');
    if (!is_string($secret)) {
      return FALSE;
    }
    return $authorization === "Basic $secret";
  }

  /**
   * Get events from request.
   */
  private function getEventsFromRequest(Request $request): array {
    $response = Json::decode($request->getContent());
    if (!\is_array($response)) {
      throw new BadRequestHttpException('Invalid JSON data.');
    }
    if (!isset($response['data']['events']) || !\is_array($response['data']['events'])) {
      throw new BadRequestHttpException('No events');
    }
    $events = $response['data']['events'];
    \usort($events, static fn (array $a, array $b) => $a['published'] <=> $b['published']);
    return $events;
  }

}
