<?php

declare(strict_types=1);

namespace Drupal\oktaplus\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\ConfigTarget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\Enum\UserAddAction;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Oktaplus settings form.
 */
final class OktaplusSettingsForm extends ConfigFormBase {

  public const string CONFIG_NAME = 'oktaplus.settings';

  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    $instance = parent::create($container);
    $instance->entityTypeManager = $container->get(EntityTypeManagerInterface::class);
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [self::CONFIG_NAME];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return __CLASS__;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $emptyToNull = static fn (string $value) => $value ?: NULL;
    $form = parent::buildForm($form, $form_state);
    $form['client'] = [
      '#type' => 'select',
      '#title' => $this->t('OpenID Connect Client'),
      '#config_target' => new ConfigTarget(
        self::CONFIG_NAME,
        'client',
        toConfig: $emptyToNull,
      ),
      '#options' => $this->getClientOptions(),
      '#empty_option' => $this->t('- None -'),
    ];
    $form['app_instance_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('AppInstance ID'),
      '#config_target' => new ConfigTarget(
        self::CONFIG_NAME,
        'app_instance_id',
        toConfig: $emptyToNull,
      ),
      '#description' => $this->t('The Okta AppInstance ID. This is required to filter application-specific events, such as <code>application.user_membership.add</code> and <code>application.user_membership.remove</code>.'),
    ];
    $form['event_hook_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Event hook path'),
      '#config_target' => new ConfigTarget(
        static::CONFIG_NAME,
        'event_hook_path',
        toConfig: $emptyToNull,
      ),
      '#description' => $this->t('The path to the Okta event hook endpoint, for example <code>/okta/event-hook</code>.'),
    ];
    $form['event_hook_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Event hook secret'),
      '#config_target' => new ConfigTarget(
        self::CONFIG_NAME,
        'event_hook_secret',
        toConfig: $emptyToNull,
      ),
      '#description' => $this->t('For basic auth, this should be the Base64-encoded username:password string, for example <code>admin:password</code> becomes <code>YWRtaW46cGFzc3dvcmQ=</code>.'),
    ];
    $form['events'] = [
      '#type' => 'details',
      '#title' => $this->t('Events'),
      '#open' => TRUE,
    ];
    $form['events'][EventType::ApplicationUserMembershipAdd->name] = [
      '#title' => $this->t('User assigned to app (@event)', ['@event' => EventType::ApplicationUserMembershipAdd->value]),
      '#type' => 'radios',
      '#options' => UserAddAction::asOptions(),
      '#required' => TRUE,
      '#config_target' => new ConfigTarget(
        self::CONFIG_NAME,
        'events.' . strtr(EventType::ApplicationUserMembershipAdd->value, '.', '_'),
      ),
    ];
    return $form;
  }

  /**
   * Gets client options.
   */
  protected function getClientOptions(): array {
    $clients = $this->entityTypeManager
      ->getStorage('openid_connect_client')
      ->loadMultiple();
    return array_map(static fn ($client) => $client->label(), $clients);
  }

}
