<?php

declare(strict_types=1);

namespace Drupal\oktaplus\Plugin\QueueWorker;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\Attribute\QueueWorker;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\externalauth\Exception\ExternalAuthRegisterException;
use Drupal\oktaplus\Enum\EventOutcome;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\Enum\UserAddAction;
use Drupal\oktaplus\Exception\InvalidLogEventException;
use Drupal\oktaplus\Exception\UnhandledLogEventTypeException;
use Drupal\oktaplus\Form\OktaplusSettingsForm;
use Drupal\oktaplus\OktaLogEvent;
use Drupal\oktaplus\OktaplusUserUtility;
use Drupal\user\UserInterface;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Event queue worker.
 */
#[QueueWorker(
  id: self::PLUGIN_ID,
  title: new TranslatableMarkup("Oktaplus event queue"),
)]
final class OktaplusEventQueue extends QueueWorkerBase implements ContainerFactoryPluginInterface, LoggerAwareInterface {

  use LoggerAwareTrait;

  public const string PLUGIN_ID = 'oktaplus_event_queue';

  public function __construct(
    array $configuration,
    string $plugin_id,
    array $plugin_definition,
    protected readonly OktaplusUserUtility $oktaplus,
    protected readonly ConfigFactoryInterface $configFactory,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get(OktaplusUserUtility::class),
      $container->get(ConfigFactoryInterface::class),
    );
    $logger = $container->get('logger.factory')->get('oktaplus');
    $instance->setLogger($logger);
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data): void {
    try {
      $event = OktaLogEvent::fromArray($data);
      if ($event->eventOutcome !== EventOutcome::Success) {
        $this->logger?->warning('Event outcome was not successful. Event type @eventType. Event UUID: @uuid. Reason: @reason.', [
          '@eventType' => $event->eventType->value,
          '@uuid' => $event->uuid,
          '@reason' => $event->payload['outcome']['reason'] ?? 'Unknown',
        ]);
        return;
      }
      // @todo this is getting unwieldy, consider refactoring handlers.
      match ($event->eventType) {
        EventType::ApplicationUserMembershipAdd => $this->handleUserMembershipAdd($event),
        EventType::UserLifecycleDeactivate,
        EventType::UserLifecycleSuspend => $this->handleUserDeactivated($event),
        EventType::ApplicationUserMembershipRemove => $this->handleUserMembershipRemove($event),
        EventType::UserLifecycleDeleteInitiated => $this->handleUserDeleted($event),
        EventType::GroupUserMembershipAdd => $this->handleUserAddedToGroup($event),
        EventType::GroupUserMembershipRemove => $this->handleUserRemovedFromGroup($event),
        default => NULL,
      };
    }
    catch (InvalidLogEventException | UnhandledLogEventTypeException $e) {
      // If we don't handle the log event type we can just continue.
      if ($e instanceof UnhandledLogEventTypeException) {
        return;
      }
      // Otherwise we should log it and do something about it.
      $this->logger?->error($e->getMessage());
    }
  }

  /**
   * Handle application membership add event.
   *
   * This will only create the user if the target app instance matches the
   * configured app instance and Oktaplus is configured to do so.
   */
  protected function handleUserMembershipAdd(OktaLogEvent $event): void {
    if (!$this->targetAppIsApplicable($event)) {
      return;
    }

    $action = $this->getApplicationUserMembershipAddAction();
    if ($action === UserAddAction::Ignore) {
      return;
    }

    // If the user doesn't exist, create it now.
    $user = $this->oktaplus->getTargetUser($event);
    if ($user === NULL) {
      try {
        $user = $this->oktaplus->createUserFromEvent($event);
      }
      catch (ExternalAuthRegisterException $e) {
        $this->logger?->error('Received @eventType event hook for @mail but unable to create user. Event uuid @uuid. Reason: @reason', [
          '@mail' => $event->getTargetUser()['alternateId'] ?? '',
          '@eventType' => $event->eventType->value,
          '@uuid' => $event->uuid,
          '@reason' => $e->getMessage(),
        ]);
        return;
      }
    }

    // @todo remove after https://www.drupal.org/project/openid_connect/issues/3504426
    assert($user instanceof UserInterface);

    // If we need to activate the user, do it now.
    if ($action === UserAddAction::Activate) {
      $this->oktaplus->activateUser($user, $event);
    }

    $this->logger?->notice('User @uid (@mail) was @action via Okta event hook. Event type @eventType. Event uuid @uuid.', [
      '@uid' => $user->id(),
      '@action' => $action->pastTense(),
      '@mail' => $user->getEmail(),
      '@eventType' => $event->eventType->value,
      '@uuid' => $event->uuid,
    ]);
  }

  /**
   * Handle application membership remove event.
   *
   * This will only deactivate the user if the target app instance matches the
   * configured app instance.
   */
  protected function handleUserMembershipRemove(OktaLogEvent $event): void {
    if (!$this->targetAppIsApplicable($event)) {
      return;
    }
    $this->handleUserDeactivated($event);
  }

  /**
   * Deactivate user.
   */
  protected function handleUserDeactivated(OktaLogEvent $event): void {
    $user = $this->oktaplus->getTargetUser($event);
    if ($user === NULL) {
      return;
    }

    $this->oktaplus->blockUser($user, $event);
    $this->logger?->notice('User @uid (@mail) was blocked via Okta event hook. Event type @eventType. Event uuid @uuid.', [
      '@uid' => $user->id(),
      '@mail' => $user->getEmail(),
      '@eventType' => $event->eventType->value,
      '@uuid' => $event->uuid,
    ]);
  }

  /**
   * Delete user from Okta.
   *
   * We only want to deactivate the user in Drupal, but we will also remove
   * their authmap entry.
   */
  protected function handleUserDeleted(OktaLogEvent $event): void {
    $this->handleUserDeactivated($event);
    $user = $this->oktaplus->getTargetUser($event);
    if ($user === NULL) {
      return;
    }
    $this->oktaplus->deleteUserFromAuthmap($user);
  }

  /**
   * User added to group.
   *
   * Assign roles to existing user if there is a matching role mapping in the
   * OpenID Connect config.
   */
  protected function handleUserAddedToGroup(OktaLogEvent $event): void {
    $group = $event->getTargetGroup();
    if ($group === NULL || !isset($group['displayName'])) {
      return;
    }

    $roles = $this->getMappedRolesForGroup($group['displayName']);
    if ($roles === []) {
      return;
    }

    $user = $this->oktaplus->getTargetUser($event);
    if ($user === NULL) {
      return;
    }

    $this->oktaplus->addUserRoles($user, $roles, $event);
    $this->logger?->notice('User @uid (@mail) had role(s) (@roles) added via Okta event hook. Event type @eventType. Event uuid @uuid.', [
      '@uid' => $user->id(),
      '@mail' => $user->getEmail(),
      '@roles' => implode(', ', $roles),
      '@eventType' => $event->eventType->value,
      '@uuid' => $event->uuid,
    ]);
  }

  /**
   * User removed from group.
   *
   * Remove roles from existing user if there is a matching role mapping in the
   * OpenID Connect config. Note that this assumes a one-to-one mapping of
   * roles to groups.
   */
  protected function handleUserRemovedFromGroup(OktaLogEvent $event): void {
    $group = $event->getTargetGroup();
    if ($group === NULL || !isset($group['displayName'])) {
      return;
    }

    $roles = $this->getMappedRolesForGroup($group['displayName']);
    if ($roles === []) {
      return;
    }

    $user = $this->oktaplus->getTargetUser($event);
    if ($user === NULL) {
      return;
    }

    $this->oktaplus->removeUserRoles($user, $roles, $event);
    $this->logger?->notice('User @uid (@mail) had role(s) (@roles) removed via Okta event hook. Event type @eventType. Event uuid @uuid.', [
      '@uid' => $user->id(),
      '@mail' => $user->getEmail(),
      '@roles' => implode(', ', $roles),
      '@eventType' => $event->eventType->value,
      '@uuid' => $event->uuid,
    ]);
  }

  /**
   * Check if the target app matches the configured app.
   */
  protected function targetAppIsApplicable(OktaLogEvent $event): bool {
    $config = $this->configFactory->get(OktaPlusSettingsForm::CONFIG_NAME);
    $configuredAppInstance = $config->get('app_instance_id');
    if ($configuredAppInstance === NULL) {
      return FALSE;
    }
    $targetAppInstance = $event->getTargetAppInstance() ?? ['id' => NULL];
    return $targetAppInstance['id'] === $configuredAppInstance;
  }

  /**
   * Determine what action to take when user is added to app.
   */
  protected function getApplicationUserMembershipAddAction(): UserAddAction {
    $config = $this->configFactory->get(OktaPlusSettingsForm::CONFIG_NAME);
    $events = $config->get('events');
    return UserAddAction::from($events['application_user_membership_add'] ?? UserAddAction::Ignore->value);
  }

  /**
   * Get the mapped role from the group.
   *
   * @todo this might be better in a utility class.
   */
  protected function getMappedRolesForGroup(string $group): array {
    $config = $this->configFactory->get('openid_connect.settings');
    $roleMappings = $config->get('role_mappings');
    return array_keys(
      array_filter(
        $roleMappings,
        static fn ($groups) => in_array($group, $groups ?? [], TRUE),
      ),
    );
  }

}
