<?php

declare(strict_types=1);

namespace Drupal\Tests\oktaplus\Kernel;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;
use Drupal\KernelTests\KernelTestBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Tests routes provided by the route subscriber.
 */
final class RouteTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'oktaplus',
    'externalauth',
  ];

  protected HttpKernelInterface $httpKernel;

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();
    $this->httpKernel = $this->container->get(HttpKernelInterface::class);
    $this->container->get('config.factory')
      ->getEditable('oktaplus.settings')
      ->set('event_hook_path', '/okta/event-hook')
      ->set('event_hook_secret', 'YWRtaW46cGFzc3dvcmQ=')
      ->save();
  }

  /**
   * Tests the event hook route.
   *
   * The configurable event hook route handles GET and POST requests with
   * different controllers. The unit tests provide coverage for the controllers
   * but don't cover the different methods and headers.
   */
  public function testEventHookRoute(): void {
    $eventUri = Url::fromRoute('oktaplus.event_hook.event')->toString();
    $verifyUri = Url::fromRoute('oktaplus.event_hook.verification')->toString();
    $this->assertSame('/okta/event-hook', $eventUri);
    $this->assertSame('/okta/event-hook', $verifyUri);

    // POST request with correct headers.
    $request = Request::create($eventUri, method: 'POST', content: Json::encode(['data' => ['events' => []]]));
    $request->headers->set('Authorization', "Basic YWRtaW46cGFzc3dvcmQ=");
    $request->headers->set('Content-Type', 'application/json');
    $response = $this->httpKernel->handle($request);
    $this->assertSame(Response::HTTP_NO_CONTENT, $response->getStatusCode());

    // POST request with missing content type.
    $request = Request::create($eventUri, method: 'POST');
    $response = $this->httpKernel->handle($request);
    $this->assertSame(Response::HTTP_UNSUPPORTED_MEDIA_TYPE, $response->getStatusCode());

    // GET request with no headers.
    $request = Request::create($verifyUri);
    $response = $this->httpKernel->handle($request);
    $this->assertSame(Response::HTTP_BAD_REQUEST, $response->getStatusCode());

    // GET request with headers.
    $request = Request::create($verifyUri, parameters: ['x-okta-verification-challenge' => '123']);
    $request->headers->set('x-okta-verification-challenge', '123');
    $response = $this->httpKernel->handle($request);
    $this->assertSame(Response::HTTP_OK, $response->getStatusCode());
    $this->assertSame('{"verification":"123"}', $response->getContent());
  }

}
